"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = exports.RiffWaveReader = void 0;

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var errorRiffTag = "RIFF chunk has wrong tag.";
var errorRiffFormat = "RIFF chunk specifies invalid format";
var errorFormatId = "Format chunk id is invalid";
var unknown = "Unknown";
var errorDataId = "Data chunk id is invalid";

var RiffWaveReader =
/*#__PURE__*/
function () {
  function RiffWaveReader(reader) {
    _classCallCheck(this, RiffWaveReader);

    this.reader = reader;
  }

  _createClass(RiffWaveReader, [{
    key: "_read",
    value: function _read(offset, size) {
      var reader = this.reader;

      if (Array.isArray(reader)) {
        return new Promise(function (resolve) {
          return resolve(reader.slice(offset, offset + size + 1));
        });
      } else if (reader.constructor && reader.constructor.name === "ArrayBuffer") {
        return new Promise(function (resolve) {
          return resolve(new Int8Array(reader.slice(offset, offset + size + 1)));
        });
      }

      return reader.read(offset, size);
    }
  }, {
    key: "readSample",
    value: function readSample(channel, index) {
      var _this = this;

      return this.readChunks().then(function (_ref) {
        var format = _ref.format,
            data = _ref.data;
        var position = data.start + index * format.sampleSize + channel * format.bitsPerSample / 8;
        var size = format.bitsPerSample / 8;
        return _this._read(position, size).then(function (buffer) {
          return uint8(buffer, 0);
        });
      });
    }
  }, {
    key: "readChunks",
    value: function readChunks() {
      var _this2 = this;

      if (this._chunks) {
        return new Promise(function (resolve) {
          return resolve(_this2._chunks);
        });
      }

      return this._read(0, 44).then(function (buffer) {
        // RIFF
        var tag = ascii(buffer, 0, 4);
        if (tag !== "RIFF") throw errorRiffTag;
        var riffSize = int32(buffer, 4);
        var format = ascii(buffer, 8, 4);
        if (format !== "WAVE") errorRiffFormat;
        var riffChunk = {
          tag: tag,
          size: riffSize,
          format: format
        }; // Format

        var id = ascii(buffer, 12, 4);
        if (id !== "fmt ") throw errorFormatId;
        var formatSize = int32(buffer, 16);
        var type = int16(buffer, 20);
        var channels = int16(buffer, 22);
        var sampleRate = int32(buffer, 24);
        var byteRate = int32(buffer, 28);
        var blockAlignment = int16(buffer, 32);
        var bitsPerSample = int16(buffer, 34); // Calculations

        var typeName = type === 1 ? "PCM" : unknown;
        var sampleSize = channels * bitsPerSample / 8;
        var tagSize = 4;
        var lengthSize = 4;
        var tlvSize = tagSize + lengthSize;
        var riffChunkSize = tlvSize + 4; // WAVE

        var formatChunkSize = tlvSize + formatSize;
        var dataChunkStart = riffChunkSize + formatChunkSize;
        var formatChunk = {
          id: id,
          size: formatSize,
          type: type,
          channels: channels,
          sampleRate: sampleRate,
          byteRate: byteRate,
          blockAlignment: blockAlignment,
          bitsPerSample: bitsPerSample,
          typeName: typeName,
          sampleSize: sampleSize
        }; // Data

        var dataChunk;
        var dataId = ascii(buffer, dataChunkStart, tagSize);
        if (dataId !== "data") throw errorDataId;
        var dataSize = int32(buffer, dataChunkStart + tagSize);
        var sampleStart = dataChunkStart + tlvSize;
        var sampleCount = (dataSize - tlvSize) / (channels * bitsPerSample / 8);
        var duration = sampleCount / sampleRate;
        dataChunk = {
          id: dataId,
          size: dataSize,
          start: sampleStart,
          sampleCount: sampleCount,
          duration: duration
        };
        return _this2._chunks = {
          riff: riffChunk,
          format: formatChunk,
          data: dataChunk
        };
      });
    }
  }]);

  return RiffWaveReader;
}();

exports.RiffWaveReader = RiffWaveReader;
var _default = RiffWaveReader;
exports["default"] = _default;

var ascii = function ascii(source, position, length) {
  var value = "";

  for (var i = 0; i < length; i++) {
    value += String.fromCharCode(source[position + i]);
  }

  return value;
};

var uint8 = function uint8(source, position) {
  return littleEndianU(source, position, 1);
};

var int16 = function int16(source, position) {
  return littleEndian(source, position, 2);
};

var int32 = function int32(source, position) {
  return littleEndian(source, position, 4);
};

var littleEndianU = function littleEndianU(source, position, length) {
  return new Uint8Array(source, position, length)[0];
};

var littleEndian = function littleEndian(source, position, length) {
  var value = 0;

  for (var i = length - 1; i >= 0; i--) {
    value *= 256;
    value += source[position + i];
  }

  return value;
};
//# sourceMappingURL=index.js.map