"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = exports.Reader = void 0;

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var fs = require("fs");

var util = require("util");

var read = util.promisify(fs.read);
var errorOpeningFile = "Unable to open file";
var errorPositionOutOfRange = "Data read out of range";

var Reader =
/*#__PURE__*/
function () {
  function Reader(source) {
    _classCallCheck(this, Reader);

    this.source = source;
  }

  _createClass(Reader, [{
    key: "read",
    value: function read(offset, size) {
      var _this = this;

      return new Promise(function (resolve, reject) {
        if (typeof _this.source === "string") {
          _this._readFile(offset, size, _this.source).then(resolve)["catch"](reject);
        } else if (Buffer.isBuffer(_this.source)) {
          _this._readBuffer(offset, size, _this.source).then(resolve)["catch"](reject);
        } else if (Array.isArray(_this.source)) {
          _this._readArray(offset, size, _this.source).then(resolve)["catch"](reject);
        } else {
          reject("Unknown source: " + _this.source);
        }
      });
    }
  }, {
    key: "_readArray",
    value: function _readArray(offset, size, array) {
      return this._readBuffer(offset, size, Buffer.from(array));
    }
  }, {
    key: "_readBuffer",
    value: function _readBuffer(offset, size, buffer) {
      return new Promise(function (resolve, reject) {
        if (offset + size > buffer.length) {
          reject(errorPositionOutOfRange);
        } else {
          resolve(buffer.slice(offset, offset + size));
        }
      });
    }
  }, {
    key: "_readFile",
    value: function _readFile(offset, size, file) {
      return new Promise(function (resolve, reject) {
        fs.open(file, "r", function (openError, fileDescriptor) {
          if (openError) {
            reject(errorOpeningFile);
          } else {
            var myBuffer = Buffer.alloc(size);
            read(fileDescriptor, myBuffer, 0, size, offset).then(function (_ref) {
              var buffer = _ref.buffer,
                  bytesRead = _ref.bytesRead;
              return bytesRead < size ? reject(errorPositionOutOfRange) : resolve(buffer);
            }).then(function (buffer) {
              return new Promise(function (closeResolve, closeReject) {
                fs.close(fileDescriptor, function (closeError) {
                  closeError && closeReject(closeError);
                  closeResolve(buffer);
                });
              });
            });
          }
        });
      });
    }
  }]);

  return Reader;
}();

exports.Reader = Reader;
var _default = Reader;
exports["default"] = _default;
//# sourceMappingURL=reader.js.map